package at.krixec.rosary

import android.content.Context
import android.os.CountDownTimer
import kotlinx.coroutines.runBlocking

class AutoPray(
    private var stringState: String,
    private val context: Context
) {

    private var stopped = true
    private var animate = false
    private var timerID = 0

    fun active() = stringState != "Off"

    fun stopped() = stopped

    fun needsAnimate() = animate

    fun timerID() = timerID

    fun stop() {
        stopped = true
    }

    fun pray() {
        stopped = false
    }

    fun noAnimate() {
        animate = false
    }

    fun inc() {
        timerID++
    }

    fun nextState(): String {

        if ( stringState == "Off" || stringState == "Fast" )
            animate = true

        stringState = when ( stringState ) {
            "Off" -> "Slow"
            "Slow" -> "Normal"
            "Normal" -> "Fast"
            else -> "Off"
        }

        saveState()

        return stringState
    }

    private fun saveState() {

        val store = DataSave( context )

        runBlocking {
            store.setAutoPray( stringState )
        }
    }

    fun prayerTime( prayer: Int ): Long {

        val speed = when ( stringState ) {
            "Slow" -> 1.5f
            "Normal" -> 1f
            "Fast" -> 0.75f
            else -> 1f
        }

        // I timed myself praying them
        // at relatively slow speed
        val newPrayerTime = when ( prayer ) {
            0 -> 30 // Credo part 1
            1 -> 25 // Pater Noster
            2 -> 11 // Gloria Patri
            3 -> 14 // Fatima
            4 -> 20 // Credo part 2
            5 -> 33 // Salve Regina
            else -> 20 // Ave Maria
        }

        return ( newPrayerTime * speed ).toLong()
    }
}

// This class exists because the .cancel() and
// and recomposition don't like each other ... I think
abstract class AutoPrayTimer(
    millisInFuture: Long,
    countDownInterval: Long,
    id: Int
) : CountDownTimer( millisInFuture, countDownInterval ) {

    val iD = id
}
