package at.krixec.rosary

import android.content.Context
import androidx.datastore.core.DataStore
import androidx.datastore.preferences.core.Preferences
import androidx.datastore.preferences.core.booleanPreferencesKey
import androidx.datastore.preferences.core.edit
import androidx.datastore.preferences.core.intPreferencesKey
import androidx.datastore.preferences.core.stringPreferencesKey
import androidx.datastore.preferences.preferencesDataStore
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.map

class DataSave( private val context: Context ) {

    companion object {
        private val Context.dataStore: DataStore< Preferences > by preferencesDataStore( "virgin" )
        private val VIRGIN = booleanPreferencesKey( "virgin" )
        private val AUTOPRAY = stringPreferencesKey( "autopray" )

        // Settings
        private val LANG1  = stringPreferencesKey( "lang1" )
        private val LANG2  = stringPreferencesKey( "lang2" )
        private val LANG3  = stringPreferencesKey( "lang3" )
        private val FATIMA = booleanPreferencesKey( "fatima" )
        private val REGINA = booleanPreferencesKey( "regina" )

        // to keep the postion upon language change
        private val DECADE  = intPreferencesKey( "decade"  )
        private val BEAD    = intPreferencesKey( "bead"    )
        private val PRAYER  = intPreferencesKey( "prayer"  )
        private val CURRENT = intPreferencesKey( "current" )
    }

    val isImmaculate: Flow< Boolean > = context.dataStore.data.map { preferences ->
        preferences[ VIRGIN ] != false
    }

    val autoPrayState: Flow< String > = context.dataStore.data.map { preferences ->
        preferences[ AUTOPRAY ] ?: "Off"
    }

    val rosaryPosition: Flow< List< Int > > = context.dataStore.data.map { preferences ->
        listOf(
            preferences[ DECADE ]  ?: 0,
            preferences[ BEAD   ]  ?: 0,
            preferences[ PRAYER ]  ?: 0,
            preferences[ CURRENT ] ?: 1
        )
    }

    val wheelLangs: Flow< List< String > > = context.dataStore.data.map { preferences ->
        listOf(
            preferences[ LANG1 ] ?: "lt",
            preferences[ LANG2 ] ?: "de",
            preferences[ LANG3 ] ?: "en",
        )
    }

    val prayersActive: Flow< List< Boolean > > = context.dataStore.data.map { preferences ->
        listOf(
            preferences[ FATIMA ] ?: true,
            preferences[ REGINA ] ?: true
        )
    }

    suspend fun setNotImmaculate() {
        context.dataStore.edit { preferences ->
            preferences[ VIRGIN ] = false
        }
    }
    suspend fun setAutoPray( state: String ) {
        context.dataStore.edit { preferences ->
            preferences[ AUTOPRAY ] = state
        }
    }
    suspend fun rememberPosition( decade: Int, bead: Int, prayer: Int, current: Int ) {
        context.dataStore.edit { preferences ->
            preferences[ DECADE ]  = decade
            preferences[ BEAD   ]  = bead
            preferences[ PRAYER ]  = prayer
            preferences[ CURRENT ] = current
        }
    }
    suspend fun resetPosition() {
        context.dataStore.edit { preferences ->
            preferences[ DECADE ]  = 0
            preferences[ BEAD   ]  = 0
            preferences[ PRAYER ]  = 0
            preferences[ CURRENT ] = 1
        }
    }
    suspend fun setLangs( lang1: String, lang2: String, lang3: String ) {
        context.dataStore.edit { preferences ->
            preferences[ LANG1 ] = lang1
            preferences[ LANG2 ] = lang2
            preferences[ LANG3 ] = lang3
        }
    }
    suspend fun setFatima( to: Boolean ) {
        context.dataStore.edit { preferences ->
            preferences[ FATIMA ] = to
        }
    }
    suspend fun setSalveRegina( to: Boolean ) {
        context.dataStore.edit { preferences ->
            preferences[ REGINA ] = to
        }
    }
}
