package at.krixec.rosary

import kotlinx.coroutines.flow.first
import kotlinx.coroutines.runBlocking


// Class to deal with the differences in local rosaries
data class Language(
    val shortHand: String,
    val fullName:  String,
    val flag: String,

    // Concerning the first 3 Ave Marias
    val aveMariaTextAbove:   Boolean,
    val aveMariaTextInside:  Boolean,

    // Concerning Ave Marias in the decades
    val holyMysteriesAbove:  Boolean,
    val holyMysteriesInside: Boolean
)

fun getAvailableLanguages() = listOf( "lt", "de", "en", "sl", "fr", "nl", "pt" )

fun getNextLang( store: DataSave ): String {

    val nextLang = runBlocking {
        val tmp = store.wheelLangs.first()
        store.setLangs( tmp[ 1 ], tmp[ 2 ], tmp[ 0 ] )
        tmp[ 1 ]
    }
    return nextLang
}

fun getAllLangs() = getAvailableLanguages().map { getLanguage( it ) }

suspend fun setLanguage(
    newLang: String,
    wheelPos: Int, store:
    DataSave,
    currentLangs: List< String >
): List< String > {

    val langs = when ( wheelPos ) {
        2 -> {
            listOf(
                newLang,
                if ( currentLangs[ 1 ] != newLang ) currentLangs[ 1 ] else currentLangs[ 2 ],
                if ( currentLangs[ 0 ] != newLang ) currentLangs[ 0 ] else currentLangs[ 2 ]
            )
        }
        1 -> {
            listOf(
                if ( currentLangs[ 2 ] != newLang ) currentLangs[ 2 ] else currentLangs[ 1 ],
                newLang,
                if ( currentLangs[ 0 ] != newLang ) currentLangs[ 0 ] else currentLangs[ 1 ]
            )
        }
        else -> {
            listOf(
                if ( currentLangs[ 2 ] != newLang ) currentLangs[ 2 ] else currentLangs[ 0 ],
                if ( currentLangs[ 1 ] != newLang ) currentLangs[ 1 ] else currentLangs[ 0 ],
                newLang
            )
        }
    }
    store.setLangs( langs[ 0 ], langs[ 1 ], langs[ 2 ] )
    return listOf( langs[ 2 ], langs[ 1 ], langs[ 0 ] )
}

fun getLanguage( shortHand: String ): Language {

    return when ( shortHand ) {
        "de" -> Language(
            shortHand,
            "Deutsch",
            flag = "\uD83C\uDDE6\uD83C\uDDF9",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = true,
            holyMysteriesAbove  = false,
            holyMysteriesInside = true
        )
        "sl" -> Language(
            shortHand,
            "Slovenščina",
            flag = "\uD83C\uDDF8\uD83C\uDDEE",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = true,
            holyMysteriesAbove  = false,
            holyMysteriesInside = true
        )
        "fr" -> Language(
            shortHand,
            "Français",
            flag = "\uD83C\uDDEB\uD83C\uDDF7",
            aveMariaTextAbove   = true,
            aveMariaTextInside  = false,
            holyMysteriesAbove  = true,
            holyMysteriesInside = false
        )
        "en" -> Language(
            shortHand,
            "English",
            flag = "\uD83C\uDDEC\uD83C\uDDE7",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = false,
            holyMysteriesAbove  = true,
            holyMysteriesInside = false
        )
        "lt" -> Language(
            shortHand,
            "Latine",
            flag = "\uD83C\uDDFB\uD83C\uDDE6",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = false,
            holyMysteriesAbove  = true,
            holyMysteriesInside = false
        )
        "nl" -> Language(
            shortHand,
            "Vlaams",
            flag = "\uD83C\uDDF3\uD83C\uDDF1",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = false,
            holyMysteriesAbove  = true,
            holyMysteriesInside = false
        )
        "pt" -> Language(
            shortHand,
            "Português",
            flag = "\uD83C\uDDF5\uD83C\uDDF9",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = false,
            holyMysteriesAbove  = true,
            holyMysteriesInside = false
        )

        // The latin way
        else -> Language(
            shortHand,
            "Enochian",
            flag = "\uD83C\uDFF4\u200D☠\uFE0F",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = false,
            holyMysteriesAbove  = true,
            holyMysteriesInside = false
        )
    }
}
