package at.krixec.rosary

import android.app.LocaleManager
import android.content.Context
import android.content.Intent
import android.graphics.Path
import android.graphics.PathMeasure
import android.os.Build
import android.os.Bundle
import android.os.LocaleList
import android.view.View
import android.view.WindowManager
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.app.AppCompatDelegate
import androidx.compose.animation.core.Animatable
import androidx.compose.animation.core.EaseInOutCubic
import androidx.compose.animation.core.repeatable
import androidx.compose.animation.core.tween
import androidx.compose.foundation.Canvas
import androidx.compose.foundation.background
import androidx.compose.foundation.gestures.detectTapGestures
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.rounded.Settings
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.FilledTonalButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.geometry.Size
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.ColorFilter
import androidx.compose.ui.graphics.Shadow
import androidx.compose.ui.graphics.asComposePath
import androidx.compose.ui.graphics.drawscope.Stroke
import androidx.compose.ui.graphics.drawscope.translate
import androidx.compose.ui.graphics.vector.rememberVectorPainter
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.platform.LocalConfiguration
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.platform.LocalLifecycleOwner
import androidx.compose.ui.platform.LocalView
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.drawText
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.rememberTextMeasurer
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.core.os.LocaleListCompat
import androidx.core.view.WindowCompat
import androidx.core.view.WindowInsetsCompat
import androidx.core.view.WindowInsetsControllerCompat
import at.krixec.rosary.ui.theme.Bead
import at.krixec.rosary.ui.theme.Orange
import at.krixec.rosary.ui.theme.RosariumTheme
import at.krixec.rosary.ui.theme.Shade
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking


// Google is a joke
// https://stackoverflow.com/a/74413010
class MainActivity : AppCompatActivity() {

    private var isFirstTime = false

    override fun onCreate( savedInstanceState: Bundle? ) {
        super.onCreate( savedInstanceState )

        val insetsController =
            WindowCompat.getInsetsController(
                window,
                window.decorView
            )

        insetsController.apply {
            hide( WindowInsetsCompat.Type.statusBars() )
            hide( WindowInsetsCompat.Type.navigationBars() )
            systemBarsBehavior = WindowInsetsControllerCompat.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
        }
        WindowCompat.setDecorFitsSystemWindows( window, false )

        // finally ;-;
        // https://stackoverflow.com/a/77559537
        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.P ) {
            window.attributes.layoutInDisplayCutoutMode = WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES
        }

        enableEdgeToEdge()
        setContent {

            val context = LocalContext.current
            val store = DataSave( context )

            // asynchronous preferences are retarded
            val immaculate = runBlocking { store.isImmaculate.first() }

            val rosaryPos = runBlocking { store.rosaryPosition.first() }
            runBlocking { store.resetPosition() }

            val autoPray = runBlocking { store.autoPrayState.first() }

            val prayersActive = runBlocking { store.prayersActive.first() }
            val fatima = prayersActive[ 0 ]
            val regina = prayersActive[ 1 ]

            if ( immaculate )
                isFirstTime = true

            // I set the language to latin if it isn't any of the available ones
            val currentLang = LocalConfiguration.current.locales[ 0 ].toString().substring( 0, 2 )

            if ( currentLang !in getAvailableLanguages() ) {
                if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU ) {
                    this.getSystemService( LocaleManager::class.java )
                        .applicationLocales = LocaleList.forLanguageTags( "lt" )
                } else {
                    AppCompatDelegate.setApplicationLocales(
                        LocaleListCompat.forLanguageTags( "lt" )
                    )
                }
            } else if ( isFirstTime ) {
                if ( currentLang != "en" )
                    runBlocking {
                        store.setLangs( currentLang, "lt", "en" )
                    }
                else
                    runBlocking {
                        store.setLangs( currentLang, "lt", "de" )
                    }
                if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU ) {
                    this.getSystemService( LocaleManager::class.java )
                        .applicationLocales = LocaleList.forLanguageTags( currentLang )
                } else {
                    AppCompatDelegate.setApplicationLocales(
                        LocaleListCompat.forLanguageTags( currentLang )
                    )
                }
            } else {
                // this is for when you return from the settings
                val langs = runBlocking { store.wheelLangs.first() }
                if ( langs[ 0 ] != currentLang )
                    if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU ) {
                        this.getSystemService( LocaleManager::class.java )
                            .applicationLocales = LocaleList.forLanguageTags( langs[ 0 ] )
                    } else {
                        AppCompatDelegate.setApplicationLocales(
                            LocaleListCompat.forLanguageTags( langs[ 0 ] )
                        )
                    }
            }

            RosariumTheme {
                App( isFirstTime, store, autoPray, rosaryPos, fatima, regina, this )
            }
        }
    }
}

@Composable
private fun App(
    immaculate: Boolean,
    store: DataSave,
    autoPrayState: String,
    rosaryPos: List< Int >,
    fatima: Boolean,
    regina: Boolean,
    outerContext: Context
) {

    val scope = rememberCoroutineScope()
    val context = LocalContext.current
    val activity = ( LocalLifecycleOwner.current as AppCompatActivity )

    // Google monkeybusiness to sabotage shadows
    if ( Build.VERSION.SDK_INT <= Build.VERSION_CODES.O_MR1 )
        LocalView.current.setLayerType( View.LAYER_TYPE_SOFTWARE, null )

    val showInfoPage  = remember { mutableStateOf( immaculate ) }
    val configuration = LocalConfiguration.current
    val screenWidth   = configuration.screenWidthDp.dp
    val screenHeight  = configuration.screenHeightDp.dp
    val density  = LocalDensity.current.density
    val screenWidthInPx  = screenWidth.value  * density
    val screenHeightInPx = screenHeight.value * density
    val beadSize = screenWidthInPx / 24f
    val knotSize = beadSize / 2f

    val progress = remember { Animatable(
        if ( immaculate ) 1.8f else 0f
    ) }
    val textMeasurer = rememberTextMeasurer()
    val autoPray = remember { AutoPray( autoPrayState, outerContext ) }
    val rosaryPosition = remember {
        RosaryPosition(
            rosaryPos[ 0 ],
            rosaryPos[ 1 ],
            rosaryPos[ 2 ],
            rosaryPos[ 3 ]
        )
    }

    @Composable
    fun InfoPage( showPage: Boolean, onChange: ( Boolean ) -> Unit ) {

        val autoPrayText = remember {
            mutableStateOf( getAutoPrayText( context, autoPrayState ) )
        }
        val settingsPath1 = remember { Path() }
        val settingsPath2 = remember { Path() }
        val painter = rememberVectorPainter( Icons.Rounded.Settings )
        var fontSize22 by remember { mutableStateOf( 22.sp ) }
        var fontSize20 by remember { mutableStateOf( 20.sp ) }
        var fontSize18 by remember { mutableStateOf( 18.sp ) }

        if ( showPage ) {
            Box(
                modifier = Modifier
                    .fillMaxSize()
                    .background( Shade )
                    .pointerInput( Unit ) {

                        detectTapGestures {
                            scope.launch {

                                if ( immaculate )
                                    store.setNotImmaculate()

                                onChange( false )
                                progress.animateTo(
                                    0f,
                                    animationSpec = repeatable(
                                        iterations = 1,
                                        animation = tween(
                                            durationMillis = 1200,
                                            easing = EaseInOutCubic
                                        )
                                    )
                                )
                            }
                        }
                    }
            ) {

                Canvas(
                    modifier = Modifier
                        .background( Color.Transparent )
                        .fillMaxSize()
                ) {

                    val x0 = screenWidthInPx + 4.dp.toPx()
                    val y0 = screenHeightInPx / 6f

                    settingsPath1.reset()
                    settingsPath1.moveTo( x0, y0 )
                    settingsPath1.quadTo(
                        screenWidthInPx / 1.15f,
                        screenHeightInPx / 5.8f,
                        screenWidthInPx / 1.1f,
                        screenHeightInPx / 4.5f
                    )

                    val pos = FloatArray( 2 )
                    val tan = FloatArray( 2 )
                    PathMeasure().apply {
                        setPath( settingsPath1, false )
                        getPosTan( length, pos, tan )
                    }

                    settingsPath2.reset()
                    settingsPath2.moveTo( pos[ 0 ] - 1.dp.toPx(), pos[ 1 ] - 1.dp.toPx() )
                    settingsPath2.quadTo(
                        screenWidthInPx / 1.05f,
                        screenHeightInPx / 4f,
                        screenWidthInPx + 4.dp.toPx(),
                        screenHeightInPx / 3f
                    )

                    val pos1 = FloatArray( 2 )
                    val tan1 = FloatArray( 2 )
                    PathMeasure().apply {
                        setPath( settingsPath2, false )
                        getPosTan( length * 0.1f, pos1, tan1 )
                    }
                    drawPath(
                        color = Color.Gray,
                        path = settingsPath1.asComposePath(),
                        style = Stroke(
                            width = knotSize
                        )
                    )
                    translate( pos1[ 0 ] - 36.dp.toPx(), pos1[ 1 ] - 36.dp.toPx() ) {
                        with( painter ) {
                            draw(
                                size = Size( 72.dp.toPx(), 72.dp.toPx() ),
                                colorFilter = ColorFilter.tint( Bead )
                            )
                        }
                    }
                    drawPath(
                        color = Color.Gray,
                        path = settingsPath2.asComposePath(),
                        style = Stroke(
                            width = knotSize
                        )
                    )
                    drawCircle(
                        radius = knotSize,
                        color = Color.Gray,
                        center = Offset( pos1[ 0 ], pos1[ 1 ] )
                    )
                }
                Button(
                    modifier = Modifier
                        .size( 120.dp )
                        .align( Alignment.TopEnd )
                        .offset( 20.dp, screenHeight / 6 ),
                    colors = ButtonDefaults.buttonColors( Color.Transparent ),
                    onClick = {
                        if ( immaculate )
                            runBlocking {
                                store.setNotImmaculate()
                            }

                        val intent = Intent( context, SettingsActivity::class.java )
                        intent.setFlags( Intent.FLAG_ACTIVITY_CLEAR_TOP )
                        context.startActivity(
                            intent
                        )
                        activity.finish()
                    }
                ) { }
                Column(
                    modifier = Modifier
                        .padding( top = screenHeight / 14 )
                        .fillMaxSize(),
                    horizontalAlignment = Alignment.CenterHorizontally,
                    verticalArrangement = Arrangement
                        .spacedBy( screenHeight / 12, Alignment.Top ),
                ) {

                    Row {

                        Text(
                            color = Orange,
                            fontSize = fontSize22,
                            fontWeight = FontWeight.ExtraBold,
                            textAlign = TextAlign.Center,
                            text = stringResource( R.string.change_lang ),
                            onTextLayout = {
                                if ( it.didOverflowWidth ) {
                                    fontSize22 *= .95f
                                }
                            }
                        )
                        Text(
                            color = Orange,
                            fontSize = fontSize22,
                            fontWeight = FontWeight.ExtraBold,
                            textAlign = TextAlign.Center,
                            text = " →",
                            onTextLayout = {
                                if ( it.didOverflowWidth ) {
                                    fontSize22 *= .95f
                                }
                            },
                            modifier = Modifier
                                .rotate( -20f )
                                .offset( 0.dp, -5.sp.value.dp )
                        )
                    }
                    Row {

                        Text(
                            color = Orange,
                            fontSize = fontSize22,
                            fontWeight = FontWeight.ExtraBold,
                            textAlign = TextAlign.Center,
                            text = stringResource( R.string.settings ),
                            onTextLayout = {
                                if ( it.didOverflowWidth ) {
                                    fontSize22 *= .95f
                                }
                            }
                        )
                        Text(
                            color = Orange,
                            fontSize = fontSize22,
                            fontWeight = FontWeight.ExtraBold,
                            textAlign = TextAlign.Center,
                            text = " →",
                            onTextLayout = {
                                if ( it.didOverflowWidth ) {
                                    fontSize22 *= .95f
                                }
                            },
                            modifier = Modifier
                                .rotate( 8f )
                                .offset( 0.dp, -5.sp.value.dp )
                        )
                    }
                }

                Column(
                    modifier = Modifier
                        .padding( bottom = screenHeight / 7 )
                        .fillMaxSize(),
                    verticalArrangement = Arrangement
                        .spacedBy( screenHeight / 7, Alignment.Bottom ),
                    horizontalAlignment = Alignment.CenterHorizontally
                ) {

                    Row(
                        modifier = Modifier
                            .width( screenWidth / 1.5f )
                            .padding( 0.dp, screenHeight / 14 ),
                        horizontalArrangement = Arrangement.SpaceEvenly,

                        ) {
                        FilledTonalButton(
                            onClick = {
                                autoPrayText.value = getAutoPrayText(
                                    context,
                                    autoPray.nextState()
                                )
                            }
                        ) {
                            Text(
                                text = "AutoPray",
                                fontWeight = FontWeight.ExtraBold,
                                fontSize = fontSize18,
                                onTextLayout = {
                                    if ( it.didOverflowWidth ) {
                                        fontSize18 *= .95f
                                    }
                                }
                            )
                        }
                        Text(
                            modifier = Modifier
                                .offset( 0.dp, 11.dp ),
                            color = Color.White,
                            text = autoPrayText.value,
                            fontWeight = FontWeight.ExtraBold,
                            fontSize = fontSize20,
                            onTextLayout = {
                                if ( it.didOverflowWidth ) {
                                    fontSize20 *= .95f
                                }
                            }
                        )
                    }

                    Text(
                        color = Orange,
                        fontSize = fontSize22,
                        fontWeight = FontWeight.ExtraBold,
                        textAlign = TextAlign.Center,
                        text = stringResource( R.string.tap_to_proceed ) + "\n►►►",
                        onTextLayout = {
                            if ( it.didOverflowWidth ) {
                                fontSize22 *= .95f
                            }
                        }
                    )

                    Text(
                        color = Orange,
                        fontSize = fontSize22,
                        fontWeight = FontWeight.ExtraBold,
                        textAlign = TextAlign.Center,
                        text = stringResource( R.string.swipe_to_go_back ) + "\n◄◄◄",
                        onTextLayout = {
                            if ( it.didOverflowWidth ) {
                                fontSize22 *= .95f
                            }
                        }
                    )
                }
            }
        }
    }

    Rosary( screenWidth, screenHeight, autoPray, rosaryPosition, fatima, regina )
    InfoPage( showInfoPage.value, onChange = { showInfoPage.value = it } )
    LanguageWheel( screenWidth, density, rosaryPosition, store, immaculate )

    val currentView = LocalView.current
    Button(
        modifier = Modifier
            .offset( -( 16.dp ), -( 10.dp ) )
            .size( 86.dp ),
        contentPadding = PaddingValues( 1.dp ),
        colors = ButtonDefaults.buttonColors( Color.Transparent ),

        onClick = {
            currentView.keepScreenOn = false
            autoPray.stop()
            autoPray.inc()
            scope.launch {
                progress.animateTo(
                    1.8f,
                    animationSpec = repeatable(
                        iterations = 1,
                        animation = tween(
                            durationMillis = 1200,
                            easing = EaseInOutCubic
                        )
                    )
                )
                showInfoPage.value = true
            }
        }
    ) { }

    Canvas(
        modifier = Modifier
            .background( Color.Transparent )
            .size( screenWidth, screenHeight )
    ) {
        val vertPadding = screenWidthInPx  / 12f
        val horiPadding = screenHeightInPx / 35f

        drawLine(
            color = Color.Gray,
            start = Offset( vertPadding, 0f ),
            end   = Offset(
                vertPadding,
                horiPadding + 36.dp.toPx() - ( horiPadding + 40.dp.toPx() ) * progress.value ),
            strokeWidth = knotSize / 1.5f
        )
        drawCircle(
            radius = beadSize,
            color = Bead,
            center = Offset(
                vertPadding,
                horiPadding + 16.dp.toPx() - ( horiPadding + 16.dp.toPx() ) * progress.value
            )
        )
        drawCircle(
            radius = knotSize / 2f,
            color = Color.Gray,
            center = Offset(
                vertPadding,
                horiPadding + 36.dp.toPx() - ( horiPadding + 16.dp.toPx() ) * progress.value
            )
        )
        val style = TextStyle.Default.copy(
            color = Color.LightGray,
            fontWeight = FontWeight.Bold,
            fontSize = 24.sp,
            shadow = Shadow( color = Color.Black, offset = Offset( 3f, 3f ) )
        )
        val textLayoutResult = textMeasurer.measure( "☰", style )
        drawText(
            textMeasurer = textMeasurer,
            text = "☰",
            topLeft = Offset(
                vertPadding - 2f - textLayoutResult.size.width / 2,
                horiPadding + 16.dp.toPx() - textLayoutResult.size.height / 2
                        - ( horiPadding + 16.dp.toPx() ) * progress.value
            ),
            style = style
        )
    }
}
