package at.krixec.rosary

import android.app.LocaleManager
import android.content.Context
import android.content.Intent
import android.graphics.PathMeasure
import android.os.Build
import android.os.Bundle
import android.os.LocaleList
import android.view.View
import android.view.WindowManager
import androidx.activity.compose.BackHandler
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.app.AppCompatDelegate
import androidx.compose.foundation.Canvas
import androidx.compose.foundation.background
import androidx.compose.foundation.gestures.detectTapGestures
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.FilledTonalButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.geometry.Rect
import androidx.compose.ui.geometry.Size
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.Path
import androidx.compose.ui.graphics.Shadow
import androidx.compose.ui.graphics.asAndroidPath
import androidx.compose.ui.graphics.drawscope.Stroke
import androidx.compose.ui.platform.LocalConfiguration
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.platform.LocalView
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.drawText
import androidx.compose.ui.text.font.FontStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.rememberTextMeasurer
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.core.view.WindowCompat
import androidx.core.view.WindowInsetsCompat
import androidx.core.view.WindowInsetsControllerCompat
import at.krixec.rosary.ui.theme.Bead
import at.krixec.rosary.ui.theme.Orange
import at.krixec.rosary.ui.theme.SelectedBead
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.runBlocking
import androidx.compose.foundation.lazy.grid.items
import androidx.compose.runtime.getValue
import androidx.compose.runtime.setValue
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.platform.LocalLifecycleOwner
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.core.os.LocaleListCompat

class SettingsActivity : AppCompatActivity() {

    override fun onCreate( savedInstanceState: Bundle? ) {
        super.onCreate( savedInstanceState )

        val insetsController =
            WindowCompat.getInsetsController(
                window,
                window.decorView
            )

        insetsController.apply {
            hide( WindowInsetsCompat.Type.statusBars() )
            hide( WindowInsetsCompat.Type.navigationBars() )
            systemBarsBehavior = WindowInsetsControllerCompat.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
        }
        WindowCompat.setDecorFitsSystemWindows( window, false )

        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.P ) {
            window.attributes.layoutInDisplayCutoutMode = WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES
        }

        enableEdgeToEdge()
        setContent{

            val context = LocalContext.current
            val store = DataSave( context )

            val langs = runBlocking {
                store.wheelLangs.first()
            }
            val prayers = runBlocking {
                store.prayersActive.first()
            }

            val activity = ( LocalLifecycleOwner.current as AppCompatActivity )

            BackHandler {
                val intent = Intent( context, MainActivity::class.java )
                intent.setFlags( Intent.FLAG_ACTIVITY_CLEAR_TOP )
                context.startActivity(
                    intent
                )
                activity.finish()
            }

            SettingsPage( store, langs, prayers, this )
        }
    }
}

@Composable
fun SettingsPage( store: DataSave, langs: List< String >, prayers: List< Boolean >, context: Context) {

    // Google monkeybusiness to sabotage shadows
    if ( Build.VERSION.SDK_INT <= Build.VERSION_CODES.O_MR1 )
        LocalView.current.setLayerType( View.LAYER_TYPE_SOFTWARE, null )


    val circle   = remember { Path() }
    val textMeasurer = rememberTextMeasurer()

    val configuration = LocalConfiguration.current
    val screenWidth   = configuration.screenWidthDp.dp
    val screenHeight  = configuration.screenHeightDp.dp
    val density  = LocalDensity.current.density
    val screenWidthInPx  = screenWidth.value  * density
    val screenHeightInPx = screenHeight.value * density

    var selectingLanguage by remember { mutableStateOf( false ) }
    var selectedBead by remember { mutableIntStateOf( -1 ) }

    val radius = screenWidthInPx / 1.5f
    val beadSize = screenWidthInPx / 18f
    val knotSize = beadSize / 2f
    var fontSize by remember { mutableStateOf( 22.sp ) }

    var lang1 by remember { mutableStateOf( langs[ 0 ] ) }
    var lang2 by remember { mutableStateOf( langs[ 1 ] ) }
    var lang3 by remember { mutableStateOf( langs[ 2 ] ) }
    val allLangs = remember { getAllLangs() }

    var fatima by remember { mutableStateOf( prayers[ 0 ] ) }
    var regina by remember { mutableStateOf( prayers[ 1 ] ) }

    val langMap = mapOf(
        0 to getLanguage( lang3 ),
        1 to getLanguage( lang2 ),
        2 to getLanguage( lang1 )
    )

    circle.reset()
    circle.addOval(
        Rect(
            Offset( ( screenWidthInPx - radius ) / 2, screenHeightInPx / 8f ),
            Size( radius, radius )
        )
    )

    Box(
        modifier = Modifier
            .fillMaxSize()
            .pointerInput( Unit ) {

                detectTapGestures {
                    if ( selectingLanguage ) {
                        selectingLanguage = false
                        selectedBead = -1
                    }
                }
            }
    ) {
        Column {

            Canvas(
                modifier = Modifier
                    .fillMaxWidth()
                    .fillMaxHeight( 0.55f )
                    .align( Alignment.CenterHorizontally )
            ) {
                drawPath(
                    circle,
                    style = Stroke( knotSize ),
                    color = Color.Gray
                )

                for ( bead in 0..2 ) {

                    val flag = langMap[ bead ]!!.flag
                    val style = TextStyle.Default.copy(
                        fontSize = 36.sp,
                        shadow = Shadow(
                            color = Color.Black,
                            offset = Offset( knotSize / 4, knotSize / 4 )
                        )
                    )
                    val textLayoutResult = textMeasurer.measure(flag, style)

                    for ( knot in -1..1 ) {
                        val pos = FloatArray( 2 )
                        val tan = FloatArray( 2 )

                        val value = bead / 3f + knot * 0.049f
                        PathMeasure().apply {
                            setPath( circle.asAndroidPath(), false )
                            getPosTan(
                                length * if ( value > 0 ) value else 1 + value,
                                pos,
                                tan
                            )
                        }

                        if ( knot == 0 ) {
                            drawCircle(
                                radius = beadSize * 1.3f,
                                color = if ( bead == selectedBead ) SelectedBead else Bead,
                                center = Offset( pos[ 0 ], pos[ 1 ] )
                            )
                            val offsetMod = offsetMod( bead, knotSize )
                            drawText(
                                textMeasurer = textMeasurer,
                                text = ( ( bead - 2 ) * -1 + 1 ).toString(),
                                topLeft = Offset(
                                    pos[ 0 ] + offsetMod[ 0 ],
                                    pos[ 1 ] + offsetMod[ 1 ]
                                ),
                                style = TextStyle.Default.copy(
                                    color = Orange,
                                    fontStyle = FontStyle.Italic,
                                    fontWeight = FontWeight.Bold,
                                    fontSize = 28.sp
                                )
                            )
                            drawText(
                                textMeasurer = textMeasurer,
                                text = flag,
                                style = style,
                                topLeft = Offset(
                                    pos[ 0 ] - textLayoutResult.size.width  / 2 - knotSize / 16,
                                    pos[ 1 ] - textLayoutResult.size.height / 2 - knotSize / 12
                                )
                            )

                        } else {
                            drawCircle(
                                radius = knotSize,
                                color = Color.Gray,
                                center = Offset( pos[ 0 ], pos[ 1 ] )
                            )
                        }
                    }
                }
            }

            if ( selectingLanguage ) {
                Box(
                    modifier = Modifier
                        .fillMaxWidth()
                        .fillMaxHeight()
                        .background( Bead )
                ) {

                    LazyVerticalGrid(
                        modifier = Modifier
                            .padding( top = screenHeight / 25 ),
                        columns = GridCells.FixedSize( screenWidth / 3 )
                    ) {
                        items( allLangs ) { lang ->
                            Column(
                                Modifier.size( screenWidth / 3 )
                            ) {
                                Text(
                                    lang.flag,
                                    Modifier.width( screenWidth / 3 ),
                                    textAlign = TextAlign.Center,
                                    fontSize = 36.sp
                                )
                                Text(
                                    lang.fullName,
                                    Modifier.width( screenWidth / 3 ),
                                    textAlign = TextAlign.Center,
                                    fontSize = fontSize,
                                    onTextLayout = {
                                        if ( it.didOverflowWidth ) {
                                            fontSize *= .95f
                                        }
                                    }
                                )
                            }
                            Button(
                                modifier = Modifier
                                    .size( screenWidth / 3 ),
                                colors = ButtonDefaults.buttonColors( Color.Transparent ),
                                onClick = {
                                    if ( langMap[ selectedBead ]!!.shortHand != lang.shortHand ) {

                                        runBlocking {
                                            val newLangs = setLanguage(
                                                lang.shortHand,
                                                selectedBead,
                                                store,
                                                listOf( lang3, lang2, lang1 )
                                            )

                                            lang1 = newLangs[ 2 ]
                                            lang2 = newLangs[ 1 ]
                                            lang3 = newLangs[ 0 ]
                                        }
                                        if ( lang.shortHand == lang1 ) {
                                            if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU ) {
                                                context.getSystemService( LocaleManager::class.java )
                                                    .applicationLocales = LocaleList.forLanguageTags( lang1 )
                                            } else {
                                                AppCompatDelegate.setApplicationLocales(
                                                    LocaleListCompat.forLanguageTags( lang1 )
                                                )
                                            }
                                        }
                                    }

                                    selectedBead = -1
                                    selectingLanguage = false
                                }
                            ) { }
                        }
                    }
                }
            } else {
                FilledTonalButton(
                    modifier = Modifier
                        .align( Alignment.CenterHorizontally ),
                    colors = ButtonDefaults.buttonColors( if ( fatima ) Bead else Color.Gray ),
                    onClick = {
                        fatima = !fatima
                        runBlocking {
                            store.setFatima( fatima )
                        }
                    }
                ) {
                    Text(
                        stringResource( R.string.settings_fatima ),
                        fontSize = 20.sp,
                        fontWeight = FontWeight.Bold
                    )
                }
                FilledTonalButton(
                    modifier = Modifier
                        .padding( 0.dp, screenHeight / 14 )
                        .align( Alignment.CenterHorizontally ),
                    colors = ButtonDefaults.buttonColors( if ( regina ) Bead else Color.Gray ),
                    onClick = {
                        regina = !regina
                        runBlocking {
                            store.setSalveRegina( regina )
                        }
                    }
                ) {
                    Text(
                        stringResource( R.string.settings_salve_regina ),
                        fontSize = 20.sp,
                        fontWeight = FontWeight.Bold
                    )
                }
            }
        }

        for ( bead in 0..2 ) {
            val pos = FloatArray( 2 )
            val tan = FloatArray( 2 )

            val value = bead / 3f
            PathMeasure().apply {
                setPath( circle.asAndroidPath(), false )
                getPosTan(
                    length * if ( value > 0 ) value else 1 + value,
                    pos,
                    tan
                )
            }
            Button(
                modifier = Modifier
                    .size( ( beadSize * 1.5f ).toDp() * 2 )
                    .offset(
                        ( pos[ 0 ] - beadSize * 1.5f ).toDp(),
                        ( pos[ 1 ] - beadSize * 1.5f ).toDp()
                    ),
                colors = ButtonDefaults.buttonColors( Color.Transparent ),
                onClick = {
                    if ( selectedBead == bead || selectedBead == -1 ) {
                        selectingLanguage = !selectingLanguage
                        selectedBead = if ( selectingLanguage ) bead else -1
                    } else
                        selectedBead = bead
                }
            ) { }
        }
    }
}