package at.krixec.rosary

import android.content.Context
import androidx.compose.runtime.Composable
import androidx.compose.ui.platform.LocalConfiguration
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.buildAnnotatedString
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.withStyle
import at.krixec.rosary.ui.theme.Orange
import java.time.LocalDate


/* https://www.baeldung.com/java-determine-easter-date-specific-year */
fun getEaster( year: Int ): LocalDate? {
    val a = year % 19
    val b = year / 100
    val c = year % 100
    val d = b / 4
    val e = b % 4
    val f = ( b + 8 ) / 25
    val g = ( b - f + 1 ) / 3
    val h = ( 19 * a + b - d - g + 15 ) % 30
    val i = c / 4
    val k = c % 4
    val l = ( 2 * e + 2 * i - h - k + 32 ) % 7
    val m = ( a + 11 * h + 22 * l ) / 451
    val t = h + l - 7 * m + 114
    val n = t / 31
    val o = t % 31

    return LocalDate.of( year, n, o + 1 )
}

@Composable
fun getHolyMystery( decade: Int, bead: Int, aveMariaBonusText: Boolean ): String {

    val date = LocalDate.now()
    val dayOfWeek = date.dayOfWeek.value

    val easter = getEaster( date.year )
    val lent = easter?.minusDays( 40 )
    val christmas = LocalDate.of( date.year, 12, 25 )
    val startOfAdvent = christmas.minusDays( ( 21 + christmas.dayOfWeek.value ).toLong() )

    val gaudiosa = dayOfWeek == 1 || dayOfWeek == 6 || date == christmas ||
            ( dayOfWeek == 7 && ( date >= startOfAdvent && date < christmas ) )

    val luminosa = dayOfWeek == 4

    val dolorosa = dayOfWeek == 2 || dayOfWeek == 5 ||
            ( dayOfWeek == 7 && ( date >= lent && date < easter ) )

    val gloriosa = dayOfWeek == 3 || dayOfWeek == 7

    if ( decade == 0 && aveMariaBonusText && gaudiosa ) {
        return when ( bead ) {
            3 -> stringResource( R.string.ave_maria_gaudiosa_1 )
            4 -> stringResource( R.string.ave_maria_gaudiosa_2 )
            5 -> stringResource( R.string.ave_maria_gaudiosa_3 )
            else -> "\n"
        }
    } else if ( decade == 0 && aveMariaBonusText && luminosa ) {
        return when ( bead ) {
            3 -> stringResource( R.string.ave_maria_luminosa_1 )
            4 -> stringResource( R.string.ave_maria_luminosa_2 )
            5 -> stringResource( R.string.ave_maria_luminosa_3 )
            else -> "\n"
        }
    } else if ( decade == 0 && aveMariaBonusText && dolorosa ) {
        return when ( bead ) {
            3 -> stringResource( R.string.ave_maria_dolorosa_1 )
            4 -> stringResource( R.string.ave_maria_dolorosa_2 )
            5 -> stringResource( R.string.ave_maria_dolorosa_3 )
            else -> "\n"
        }
    } else if ( decade == 0 && aveMariaBonusText && gloriosa ) {
        return when ( bead ) {
            3 -> stringResource( R.string.ave_maria_gloriosa_1 )
            4 -> stringResource( R.string.ave_maria_gloriosa_2 )
            5 -> stringResource( R.string.ave_maria_gloriosa_3 )
            else -> "\n"
        }
    }

    return when {
        gaudiosa -> when ( decade ) {
            1 -> stringResource( R.string.gaudiosa_1 )
            2 -> stringResource( R.string.gaudiosa_2 )
            3 -> stringResource( R.string.gaudiosa_3 )
            4 -> stringResource( R.string.gaudiosa_4 )
            5 -> stringResource( R.string.gaudiosa_5 )
            else -> ""
        }
        luminosa -> when ( decade ) {
            1 -> stringResource( R.string.luminosa_1 )
            2 -> stringResource( R.string.luminosa_2 )
            3 -> stringResource( R.string.luminosa_3 )
            4 -> stringResource( R.string.luminosa_4 )
            5 -> stringResource( R.string.luminosa_5 )
            else -> ""
        }
        dolorosa -> when ( decade ) {
            1 -> stringResource( R.string.dolorosa_1 )
            2 -> stringResource( R.string.dolorosa_2 )
            3 -> stringResource( R.string.dolorosa_3 )
            4 -> stringResource( R.string.dolorosa_4 )
            5 -> stringResource( R.string.dolorosa_5 )
            else -> ""
        }
        gloriosa -> when ( decade ) {
            1 -> stringResource( R.string.gloriosa_1 )
            2 -> stringResource( R.string.gloriosa_2 )
            3 -> stringResource( R.string.gloriosa_3 )
            4 -> stringResource( R.string.gloriosa_4 )
            5 -> stringResource( R.string.gloriosa_5 )
            else -> ""
        }
        else -> ""
    }
}

@Composable
fun getPrayer( prayer: Int, decade: Int, bonusDecade: Int, bead: Int ): AnnotatedString {

    @Composable
    fun buildHailMary(): AnnotatedString {

        val currentLang = LocalConfiguration.current.locales[ 0 ].toString()
        val langDetails = getLanguage( currentLang )

        val realDecade = decade + bonusDecade

        val mystery = getHolyMystery(
            realDecade,
            bead,
            langDetails.aveMariaTextAbove || langDetails.aveMariaTextInside
        )
        return if (
            ( realDecade == 0 && langDetails.aveMariaTextAbove ) ||
            ( realDecade != 0 && langDetails.holyMysteriesAbove )
            )
            buildAnnotatedString {
                withStyle( style = SpanStyle( color = Orange, fontWeight = FontWeight.Bold ) ) {
                    append( mystery )
                }
                append( stringResource( R.string.ave_maria_1 ) )

                append( stringResource( R.string.ave_maria_2 ) )
            }
        else
            buildAnnotatedString {
                append( stringResource( R.string.ave_maria_1 ) )

                withStyle( style = SpanStyle( color = Orange, fontWeight =  FontWeight.Bold ) ) {
                    append( mystery )
                }
                append( stringResource( R.string.ave_maria_2 ) )
            }
    }

    return when ( prayer ) {
        0 -> buildAnnotatedString { append( stringResource( R.string.credo ) ) }
        1 -> buildAnnotatedString { append( stringResource( R.string.pater_noster ) ) }
        2 -> buildAnnotatedString { append( stringResource( R.string.gloria_patri ) ) }
        3 -> buildAnnotatedString { append( stringResource( R.string.fatima ) ) }
        4 -> buildAnnotatedString { append( stringResource( R.string.credo_2 ) ) }
        5 -> buildAnnotatedString { append( stringResource( R.string.salve_regina ) ) }

        else -> {
            buildHailMary()
        }
    }
}

fun getSymbol( stopped: Boolean ): String {

    return if ( stopped )
        "▶"
    else
        "■"
}

fun getAutoPrayText( context: Context, state: String ): String {

    return when ( state ) {
        "Off"    -> context.resources.getString( R.string.autopray_off    )
        "Slow"   -> context.resources.getString( R.string.autopray_slow   )
        "Normal" -> context.resources.getString( R.string.autopray_normal )
        "Fast"   -> context.resources.getString( R.string.autopray_fast   )
        else -> ""
    }
}

fun offsetMod( bead: Int, knotSize: Float ): Array< Float > {

    return when ( bead ) {
        0 -> arrayOf( -knotSize * 5.5f, -knotSize * 1.6f )
        1 -> arrayOf( knotSize * 1.8f, knotSize * 3f )
        else -> arrayOf( knotSize * 1.4f, -knotSize * 6 )
    }
}

@Composable
fun Float.toDp() = with( LocalDensity.current ) { this@toDp.toDp() }
